//
// C++ Interface: burndatadvdconfigwidget
//
// Description: 
//
//
// Author: James Thorniley <james@uncommonlygood.co.uk>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef BURNDATADVDCONFIGWIDGET_H
#define BURNDATADVDCONFIGWIDGET_H

#include <burndatacdconfigwidget.h>

class BurnSidebarPlugin;

/**
@author James Thorniley
*/
class BurnDataDVDConfigWidget : public BurnDataCDConfigWidget
{
Q_OBJECT
public:
    BurnDataDVDConfigWidget(BurnSidebarPlugin *plugin, QWidget *parent = 0, const char *name = 0);

    virtual ~BurnDataDVDConfigWidget();

    virtual K3bDoc *k3bDoc();


};

#endif
