/***************************************************************************
 *   Copyright (C) 2005 by James Thorniley                                 *
 *   james@uncommonlygood.co.uk                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BURNPROJECTTYPEWIDGET_H
#define BURNPROJECTTYPEWIDGET_H

#include <qwidget.h>
#include <klistview.h>
#include "burnsidebarplugin.h"

class QResizeEvent;
class QString;
class QPixmap;
class QLabel;
class QMouseEvent;

/**
This widget displays project types (data CD, audio CD etc), 
the user can drag it onto a CD writer device widget (BurnDeviceWidget) to initiate burning.

@author James Thorniley
*/
class BurnProjectTypeWidget : public KListView
{
  Q_OBJECT
  public:
    BurnProjectTypeWidget( bool enableDVD, QWidget *parent = 0, const char *name = 0);

    ~BurnProjectTypeWidget();

  signals:
    ///Emitted when a project type is selected
    void discTypeSelected( BurnSidebarPlugin::DiscType type );

  protected:
    /**returns a drag object containing a K3b project for the currently
    selected project*/
    virtual QDragObject *dragObject();

    //virtual void resizeEvent( QResizeEvent *);
  private:
    KListViewItem *m_dataCDItem, *m_audioCDItem, *m_dataDVDItem;

  private slots:
    void slotSelectionChanged( QListViewItem *item );
};

#endif
