/***************************************************************************
 *   Copyright (C) 2005 by James Thorniley                                 *
 *   james@uncommonlygood.co.uk                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "burninfomessagedialog.h"
#include <qtextedit.h>
#include <qstringlist.h>
#include <klocale.h>
#include <qlayout.h>

BurnInfoMessageDialog::BurnInfoMessageDialog(const QStringList &messages, QWidget *parent, const char *name)
 : KDialogBase(KDialogBase::Plain, WStyle_DialogBorder, parent, name, true, i18n("Error messages"), KDialogBase::Ok, KDialogBase::Ok, true)
{

  m_textBox = new QTextEdit( plainPage() );
  m_textBox->setText( messages.join("\n"), QString::null );
  m_textBox->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
  m_textBox->setReadOnly( true );
  new QVBoxLayout( plainPage() );
  plainPage()->layout()->add(m_textBox );
  resize( 400, 400 );
}


BurnInfoMessageDialog::~BurnInfoMessageDialog()
{
}


#include "burninfomessagedialog.moc"
