
#ifndef _burn_H_
#define _burn_H_

#include <qstring.h>
#include <qcstring.h>

#include <kurl.h>
#include <kio/global.h>
#include <kio/slavebase.h>
#include <kio/forwardingslavebase.h>

class QCString;
class QDir;
class KURL;

class kio_burnProtocol : public KIO::ForwardingSlaveBase
{
public:
    kio_burnProtocol(const QCString &protocol, const QCString &pool_socket, const QCString &app_socket);
    virtual ~kio_burnProtocol();

protected:
    virtual bool rewriteURL( const KURL &url, KURL &newURL );

  private:
    QDir *m_dataDir;

};

#endif
